@@ -47,6 +47,7 @@ struct RhythmDBPrivate
 	GStaticRWLock lock;
 
 	GType *column_types;
+	char **column_xml_names;
 
 	GThreadPool *query_thread_pool;
 
@@ -62,6 +63,7 @@ struct RhythmDBPrivate
 	GAsyncQueue *action_queue;
 
 	gboolean dry_run;
+	gboolean no_update;
 
 	gboolean writelocked;
 	guint readlocks;
@@ -78,6 +80,12 @@ struct RhythmDBPrivate
 
 	GMutex *exit_mutex;
 	gboolean exiting;
+	
+	GCond *saving_condition;
+	GMutex *saving_mutex;
+
+	gboolean saving;
+	gboolean dirty;
 };
 
 struct RhythmDBAction
@@ -131,6 +139,7 @@ enum
 	PROP_0,
 	PROP_NAME,
 	PROP_DRY_RUN,
+	PROP_NO_UPDATE,
 };
 
 enum
@@ -140,6 +149,7 @@ enum
 	ENTRY_CHANGED,
 	ENTRY_DELETED,
 	LOAD_COMPLETE,
+	SAVE_COMPLETE,
 	ERROR,
 	LAST_SIGNAL
 };
@@ -204,6 +214,13 @@ rhythmdb_class_init (RhythmDBClass *klass)
 							       "Whether or not changes should be saved",
 							       FALSE,
 							       G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_NO_UPDATE,
+					 g_param_spec_boolean ("no-update",
+							       "no update",
+							       "Whether or not to update the database",
+							       FALSE,
+							       G_PARAM_READWRITE));
 	rhythmdb_signals[ENTRY_ADDED] =
 		g_signal_new ("entry_added",
 			      RHYTHMDB_TYPE,
@@ -253,6 +270,17 @@ rhythmdb_class_init (RhythmDBClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+
+	rhythmdb_signals[SAVE_COMPLETE] =
+		g_signal_new ("save_complete",
+			      RHYTHMDB_TYPE,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RhythmDBClass, save_complete),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+
 	rhythmdb_signals[ERROR] =
 		g_signal_new ("error",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -286,6 +314,9 @@ prop_from_metadata_field (RBMetaDataField field, RhythmDBPropType *prop)
 	case RB_METADATA_FIELD_TRACK_NUMBER:
 		*prop = RHYTHMDB_PROP_TRACK_NUMBER; 
 		return TRUE;		
+	case RB_METADATA_FIELD_DISC_NUMBER:
+		*prop = RHYTHMDB_PROP_DISC_NUMBER; 
+		return TRUE;		
 	case RB_METADATA_FIELD_DURATION:
 		*prop = RHYTHMDB_PROP_DURATION; 
 		return TRUE;		
@@ -304,25 +335,28 @@ metadata_field_from_prop (RhythmDBPropType prop, RBMetaDataField *field)
 	switch (prop) {
 	case RHYTHMDB_PROP_TITLE:
 		*field = RB_METADATA_FIELD_TITLE;
-		return TRUE;		
+		return TRUE;
 	case RHYTHMDB_PROP_ARTIST:
 		*field = RB_METADATA_FIELD_ARTIST; 
-		return TRUE;		
+		return TRUE;
 	case RHYTHMDB_PROP_ALBUM:
 		*field = RB_METADATA_FIELD_ALBUM; 
-		return TRUE;		
+		return TRUE;
 	case RHYTHMDB_PROP_GENRE:
 		*field = RB_METADATA_FIELD_GENRE; 
-		return TRUE;		
+		return TRUE;
 	case RHYTHMDB_PROP_TRACK_NUMBER:
 		*field = RB_METADATA_FIELD_TRACK_NUMBER; 
-		return TRUE;		
+		return TRUE;
+	case RHYTHMDB_PROP_DISC_NUMBER:
+		*field = RB_METADATA_FIELD_DISC_NUMBER; 
+		return TRUE;
 	case RHYTHMDB_PROP_DURATION:
 		*field = RB_METADATA_FIELD_DURATION; 
-		return TRUE;		
+		return TRUE;
 	case RHYTHMDB_PROP_BITRATE:
 		*field = RB_METADATA_FIELD_BITRATE; 
-		return TRUE;		
+		return TRUE;
 	case RHYTHMDB_PROP_TRACK_GAIN:
 		*field = RB_METADATA_FIELD_TRACK_GAIN;
 		return TRUE;
@@ -369,6 +403,23 @@ extract_gtype_from_enum_entry (RhythmDB *db, GEnumClass *klass, guint i)
 	return ret;
 }
 
+static char *
+extract_nice_name_from_enum_entry (RhythmDB *db, GEnumClass *klass, guint i)
+{
+	GEnumValue *value;
+	char *name;
+	char *name_end;
+	
+	value = g_enum_get_value (klass, i);
+
+	name = strstr (value->value_nick, "[");
+	g_return_val_if_fail (name != NULL, NULL);
+	name_end = strstr (name, "]");
+	name++;
+
+	return g_strndup (name, name_end-name);
+}
+
 static void
 rhythmdb_init (RhythmDB *db)
 {
@@ -394,18 +445,23 @@ rhythmdb_init (RhythmDB *db)
 	g_assert (prop_class->n_values + unsaved_prop_class->n_values == RHYTHMDB_NUM_PROPERTIES);
 	g_assert (prop_class->n_values == RHYTHMDB_NUM_SAVED_PROPERTIES);
 	db->priv->column_types = g_new (GType, RHYTHMDB_NUM_PROPERTIES);
+	db->priv->column_xml_names = g_new (char *, RHYTHMDB_NUM_PROPERTIES);
 	
-	/* Now, extract the GType of each column from the enum descriptions,
-	 * and cache that for later use. */
+	/* Now, extract the GType and XML tag of each column from the
+	 * enum descriptions, and cache that for later use. */
 	for (i = 0; i < prop_class->n_values; i++) {
 		db->priv->column_types[i] = extract_gtype_from_enum_entry (db, prop_class, i);
 		g_assert (db->priv->column_types[i] != G_TYPE_INVALID);
+		db->priv->column_xml_names[i] = extract_nice_name_from_enum_entry (db, prop_class, i);
+		g_assert (db->priv->column_xml_names[i]);
 	}
 			
 	
 	for (j = 0; j < unsaved_prop_class->n_values; i++, j++) {
 		db->priv->column_types[i] = extract_gtype_from_enum_entry (db, unsaved_prop_class, i);
 		g_assert (db->priv->column_types[i] != G_TYPE_INVALID);
+		db->priv->column_xml_names[i] = extract_nice_name_from_enum_entry (db, unsaved_prop_class, i);
+		g_assert (db->priv->column_xml_names[i]);
 	}
 
 	g_type_class_unref (prop_class);
@@ -431,6 +487,13 @@ rhythmdb_init (RhythmDB *db)
 	g_async_queue_ref (db->priv->main_thread_cancel);
 	g_async_queue_ref (db->priv->action_queue);
 	g_thread_create ((GThreadFunc) action_thread_main, db, FALSE, NULL);
+	
+	db->priv->saving_condition = g_cond_new ();
+	db->priv->saving_mutex = g_mutex_new ();
+
+	db->priv->exiting = FALSE;
+	db->priv->saving = FALSE;
+	db->priv->dirty = FALSE;
 }
 
 gboolean
@@ -493,6 +556,9 @@ rhythmdb_finalize (GObject *object)
 
 	g_async_queue_unref (db->priv->status_queue);
 
+	g_mutex_free (db->priv->saving_mutex);
+	g_cond_free (db->priv->saving_condition);
+
 	g_free (db->priv->column_types);
 
 	g_hash_table_destroy (db->priv->propname_map);
@@ -520,6 +586,9 @@ rhythmdb_set_property (GObject *object,
 	case PROP_DRY_RUN:
 		source->priv->dry_run = g_value_get_boolean (value);
 		break;
+	case PROP_NO_UPDATE:
+		source->priv->no_update = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -542,6 +611,9 @@ rhythmdb_get_property (GObject *object,
 	case PROP_DRY_RUN:
 		g_value_set_boolean (value, source->priv->dry_run);
 		break;
+	case PROP_NO_UPDATE:
+		g_value_set_boolean (value, source->priv->no_update);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -728,6 +800,17 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry)
 	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_TRACK_NUMBER, &val);
 	g_value_unset (&val);
 
+	/* disc number */
+	if (!rb_metadata_get (db->priv->metadata,
+			      RB_METADATA_FIELD_DISC_NUMBER,
+			      &val)) {
+		g_value_init (&val, G_TYPE_INT);
+		g_value_set_int (&val, -1);
+	}
+
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_DISC_NUMBER, &val);
+	g_value_unset (&val);
+
 	/* duration */
 	if (rb_metadata_get (db->priv->metadata,
 			     RB_METADATA_FIELD_DURATION,
@@ -822,10 +905,11 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry)
 RhythmDBEntry *
 rhythmdb_add_song (RhythmDB *db, const char *uri, GError **real_error)
 {
-	RhythmDBEntry *entry;
+	RhythmDBEntry *entry = NULL;
 	char *realuri;
+	const char *mime;
 	GError *error = NULL;
-	GValue last_time = {0, }, rating = {0, };
+	GValue value = {0, };
 
 	realuri = rb_uri_resolve_symlink (uri);
 
@@ -850,8 +934,14 @@ rhythmdb_add_song (RhythmDB *db, const char *uri, GError **real_error)
 	if (error) {
 		g_propagate_error (real_error, error);
 		rb_debug ("failed to read data from \"%s\"", uri);
-		g_free (realuri);
-		return NULL;
+		goto out_freeuri;
+	}
+
+	mime = rb_metadata_get_mime (db->priv->metadata);
+
+	if (!mime) {
+		rb_debug ("unsupported file");
+		goto out_freeuri;
 	}
 
 	rhythmdb_write_lock (db);
@@ -862,21 +952,28 @@ rhythmdb_add_song (RhythmDB *db, const char *uri, GError **real_error)
 	set_props_from_metadata (db, entry);
 
 	/* initialize the last played date to 0=never */
-	g_value_init (&last_time, G_TYPE_LONG);
-	g_value_set_long (&last_time, 0);
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &last_time);
-	g_value_unset (&last_time);
+	g_value_init (&value, G_TYPE_LONG);
+	g_value_set_long (&value, 0);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &value);
+	g_value_unset (&value);
 
-	/* initialize the rating to 3 */
-	g_value_init (&rating, G_TYPE_DOUBLE);
-	g_value_set_double (&rating, 3.0);
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_RATING, &rating);
-	g_value_unset (&rating);
+	/* initialize the rating */
+	g_value_init (&value, G_TYPE_DOUBLE);
+	g_value_set_double (&value, 2.5);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_RATING, &value);
+	g_value_unset (&value);
 
+	/* initialize auto rating */
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, TRUE);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_AUTO_RATE, &value);
+	g_value_unset (&value);
+	
  out_dupentry:
+	rhythmdb_write_unlock_internal (db, FALSE);
+ out_freeuri:
 	g_free (realuri);
 
-	rhythmdb_write_unlock_internal (db, FALSE);
 	return entry;
 }
 
@@ -963,7 +1060,7 @@ add_thread_main (RhythmDB *db)
 		else
 			rb_uri_handle_recursively (uri, (GFunc) add_file,
 						   &db->priv->exiting, db);
-
+						 
 		if (error != NULL)
 			push_err (db, realuri, error);
 
@@ -1134,6 +1231,7 @@ rhythmdb_load_thread_main (RhythmDB *db)
 
 	g_signal_emit (G_OBJECT (db), rhythmdb_signals[LOAD_COMPLETE], 0);
 	g_async_queue_push (db->priv->status_queue, db);
+	
 	return NULL;
 }
 
@@ -1142,17 +1240,85 @@ rhythmdb_load (RhythmDB *db)
 {
 	g_object_ref (G_OBJECT (db));
 	db->priv->outstanding_threads++;
-	g_thread_create ((GThreadFunc) rhythmdb_load_thread_main, db, TRUE, NULL);
+	g_thread_create ((GThreadFunc) rhythmdb_load_thread_main, db, FALSE, NULL);
+}
+
+static void
+rhythmdb_save_worker (RhythmDB *db)
+{
+	RhythmDBClass *klass;
+	
+	g_mutex_lock (db->priv->saving_mutex);
+
+	if (db->priv->saving) {
+		rb_debug ("already saving, ignoring");
+		g_mutex_unlock (db->priv->saving_mutex);
+		return;
+	}
+
+	if (!db->priv->dirty) {
+		rb_debug ("no save needed, ignoring");
+		g_mutex_unlock (db->priv->saving_mutex);
+		return;
+	}
+
+	db->priv->saving = TRUE;
+
+	g_mutex_unlock (db->priv->saving_mutex);
+
+	rb_debug ("saving rhythmdb");
+			
+	klass = RHYTHMDB_GET_CLASS (db);
+	db_enter (db, FALSE);
+	klass->impl_save (db);
+
+	g_mutex_lock (db->priv->saving_mutex);
+
+	db->priv->saving = FALSE;
+	db->priv->dirty = FALSE;
+
+	g_mutex_unlock (db->priv->saving_mutex);
+
+	g_cond_broadcast (db->priv->saving_condition);
+}
+
+static gpointer
+rhythmdb_save_thread_main (RhythmDB *db)
+{
+	rb_debug ("entering save thread");
+	
+	rhythmdb_save_worker (db);
+
+	g_signal_emit (G_OBJECT (db), rhythmdb_signals[SAVE_COMPLETE], 0);
+	g_async_queue_push (db->priv->status_queue, db);
+	
+	return NULL;
 }
 
 void
 rhythmdb_save (RhythmDB *db)
 {
-	RhythmDBClass *klass = RHYTHMDB_GET_CLASS (db);
+	rb_debug ("saving the rhythmdb in the background");
 
-	db_enter (db, FALSE);
+	g_object_ref (G_OBJECT (db));
+	db->priv->outstanding_threads++;
 	
-	klass->impl_save (db);
+	g_thread_create ((GThreadFunc) rhythmdb_save_thread_main, db, FALSE, NULL);
+}
+
+void
+rhythmdb_save_blocking (RhythmDB *db)
+{
+	rb_debug("saving the rhythmdb and blocking");
+	
+	rhythmdb_save_worker (db);
+	
+	g_mutex_lock (db->priv->saving_mutex);
+
+	while (db->priv->saving)
+		g_cond_wait (db->priv->saving_condition, db->priv->saving_mutex);
+
+	g_mutex_unlock (db->priv->saving_mutex);
 }
 
 static void
@@ -1232,6 +1398,9 @@ rhythmdb_entry_set (RhythmDB *db, RhythmDBEntry *entry,
 	klass->impl_entry_set (db, entry, propid, value);
 
 	rhythmdb_entry_sync_mirrored (db, entry, propid, value);
+	
+	/* set the dirty state */
+	db->priv->dirty = TRUE;
 }
 
 void
@@ -1324,8 +1493,26 @@ rhythmdb_entry_delete (RhythmDB *db, RhythmDBEntry *entry)
 	rhythmdb_emit_entry_deleted (db, entry);
 	
 	klass->impl_entry_delete (db, entry);
+	
+	/* deleting an entry makes the db dirty */
+	db->priv->dirty = TRUE;
 }
 
+void
+rhythmdb_entry_delete_by_type (RhythmDB *db, RhythmDBEntryType type)
+{
+	RhythmDBClass *klass = RHYTHMDB_GET_CLASS (db);
+	
+	db_enter (db, TRUE);
+
+	if (klass->impl_entry_delete_by_type) {
+		klass->impl_entry_delete_by_type (db, type);
+	} else {
+		g_warning ("delete_by_type not implemented");
+	}
+}
+
+
 GPtrArray *
 rhythmdb_query_copy (GPtrArray *array)
 {
@@ -1462,67 +1649,7 @@ rhythmdb_query_free (GPtrArray *query)
 inline const char *
 rhythmdb_nice_elt_name_from_propid (RhythmDB *db, gint propid)
 {
-	switch (propid) {
-	case RHYTHMDB_PROP_TYPE:
-		return "type";
-	case RHYTHMDB_PROP_TITLE:
-		return "title";
-	case RHYTHMDB_PROP_TITLE_FOLDED:
-		return "title-folded";
-	case RHYTHMDB_PROP_TITLE_SORT_KEY:
-		return "title-sort-key";
-	case RHYTHMDB_PROP_GENRE:
-		return "genre";
-	case RHYTHMDB_PROP_GENRE_FOLDED:
-		return "genre-folded";
-	case RHYTHMDB_PROP_GENRE_SORT_KEY:
-		return "genre-sort-key";
-	case RHYTHMDB_PROP_ARTIST:
-		return "artist";
-	case RHYTHMDB_PROP_ARTIST_FOLDED:
-		return "artist-folded";
-	case RHYTHMDB_PROP_ARTIST_SORT_KEY:
-		return "artist-sort-key";
-	case RHYTHMDB_PROP_ALBUM:
-		return "album";
-	case RHYTHMDB_PROP_ALBUM_FOLDED:
-		return "album-folded";
-	case RHYTHMDB_PROP_ALBUM_SORT_KEY:
-		return "album-sort-key";
-	case RHYTHMDB_PROP_TRACK_NUMBER:
-		return "track-number";
-	case RHYTHMDB_PROP_DURATION:
-		return "duration";
-	case RHYTHMDB_PROP_FILE_SIZE:
-		return "file-size";
-	case RHYTHMDB_PROP_LOCATION:
-		return "location";
-	case RHYTHMDB_PROP_MTIME:
-		return "mtime";
-	case RHYTHMDB_PROP_RATING:
-		return "rating";
-	case RHYTHMDB_PROP_PLAY_COUNT:
-		return "play-count";
-	case RHYTHMDB_PROP_LAST_PLAYED:
-		return "last-played";
-	case RHYTHMDB_PROP_LAST_PLAYED_STR:
-		return "last-played-str";
-	case RHYTHMDB_PROP_BITRATE:
-		return "bitrate";
-	case RHYTHMDB_PROP_TRACK_GAIN:
-		return "replaygain-track-gain";
-	case RHYTHMDB_PROP_TRACK_PEAK:
-		return "replaygain-track-peak";
-	case RHYTHMDB_PROP_ALBUM_GAIN:
-		return "replaygain-album-gain";
-	case RHYTHMDB_PROP_ALBUM_PEAK:
-		return "replaygain-album-peak";
-	case RHYTHMDB_PROP_MIMETYPE:
-		return "mimetype";
-	default:
-		g_assert_not_reached ();
-	}
-	return NULL;
+	return db->priv->column_xml_names[propid];
 }
 
 inline int
@@ -1658,8 +1785,14 @@ rhythmdb_query_serialize (RhythmDB *db, GPtrArray *query,
 		case RHYTHMDB_QUERY_END:
 			break;
 		case RHYTHMDB_QUERY_PROP_GREATER:
+			subnode = xmlNewChild (node, NULL, "greater", NULL);
+			xmlSetProp (subnode, "prop", rhythmdb_nice_elt_name_from_propid (db, data->propid));
+			write_encoded_gvalue (subnode, data->val);
+			break;
 		case RHYTHMDB_QUERY_PROP_LESS:
-			g_assert_not_reached ();
+			subnode = xmlNewChild (node, NULL, "less", NULL);
+			xmlSetProp (subnode, "prop", rhythmdb_nice_elt_name_from_propid (db, data->propid));
+			write_encoded_gvalue (subnode, data->val);
 			break;
 		}		
 	}
@@ -1697,12 +1830,18 @@ rhythmdb_query_deserialize (RhythmDB *db, xmlNodePtr parent)
 			data->type = RHYTHMDB_QUERY_PROP_NOT_LIKE;
 		} else if (!strcmp (child->name, "equals")) {
 			data->type = RHYTHMDB_QUERY_PROP_EQUALS;
+		} else if (!strcmp (child->name, "greater")) {
+			data->type = RHYTHMDB_QUERY_PROP_GREATER;
+		} else if (!strcmp (child->name, "less")) {
+			data->type = RHYTHMDB_QUERY_PROP_LESS;
 		} else
  			g_assert_not_reached ();
 
 		if (!strcmp (child->name, "like")
 		    || !strcmp (child->name, "not-like")
-		    || !strcmp (child->name, "equals")) {
+		    || !strcmp (child->name, "equals")
+		    || !strcmp (child->name, "greater")
+		    || !strcmp (child->name, "less")) {
 			char *propstr = xmlGetProp (child, "prop");
 			gint propid = rhythmdb_propid_from_nice_elt_name (db, propstr);
 			g_free (propstr);
@@ -1906,29 +2045,40 @@ rhythmdb_prop_get_type (void)
 	{
 		static const GEnumValue values[] =
 		{
+			/* We reuse the description to store extra data about
+			* a property.  The first part is just a generic
+			* human-readable description.  Next, there is optionally
+			* a string describing the GType of the property, in
+			* parenthesis.  If this doesn't exist, we assume
+			* the property is mirrored from RBMetaData, and
+			* get the type of the property from there.
+			* Finally, there is the XML element name in brackets.
+			*/
+			ENUM_ENTRY (RHYTHMDB_PROP_TYPE, "Type of entry (gint) [type]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_TITLE, "Title [title]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_GENRE, "Genre [genre]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_ARTIST, "Artist [artist]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM, "Album [album]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_TRACK_NUMBER, "Track Number [track-number]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_DISC_NUMBER, "Disc Number [disc-number]"),
 
-			ENUM_ENTRY (RHYTHMDB_PROP_TYPE, "Type of entry (gint)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_TITLE, "Title"),
-			ENUM_ENTRY (RHYTHMDB_PROP_GENRE, "Genre"),
-			ENUM_ENTRY (RHYTHMDB_PROP_ARTIST, "Artist"),
-			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM, "Album"),
-			ENUM_ENTRY (RHYTHMDB_PROP_TRACK_NUMBER, "Track Number"),
-			ENUM_ENTRY (RHYTHMDB_PROP_DURATION, "Duration"),
-			ENUM_ENTRY (RHYTHMDB_PROP_FILE_SIZE, "File Size (guint64)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_LOCATION, "Location (gchararray)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_MTIME, "Modification time (glong)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_RATING, "Rating (gdouble)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_PLAY_COUNT, "Play Count (gint)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_LAST_PLAYED, "Last Played (glong)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_BITRATE, "Bitrate"),
-			ENUM_ENTRY (RHYTHMDB_PROP_TRACK_GAIN, "Replaygain track gain"),
-			ENUM_ENTRY (RHYTHMDB_PROP_TRACK_PEAK, "Replaygain track peak"),
-			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM_GAIN, "Replaygain album pain"),
-			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM_PEAK, "Replaygain album peak"),
-			ENUM_ENTRY (RHYTHMDB_PROP_MIMETYPE, "Mime Type (gchararray)"),
+			ENUM_ENTRY (RHYTHMDB_PROP_DURATION, "Duration [duration]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_FILE_SIZE, "File Size (guint64) [file-size]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_LOCATION, "Location (gchararray) [location]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_MTIME, "Modification time (glong) [mtime]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_RATING, "Rating (gdouble) [rating]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_AUTO_RATE, "Whether to auto-rate song (gboolean) [auto-rate]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_PLAY_COUNT, "Play Count (gint) [play-count]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_LAST_PLAYED, "Last Played (glong) [last-played]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_BITRATE, "Bitrate [bitrate]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_TRACK_GAIN, "Replaygain track gain [replaygain-track-gain]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_TRACK_PEAK, "Replaygain track peak [replaygain-track-peak]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM_GAIN, "Replaygain album pain [replaygain-album-gain]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM_PEAK, "Replaygain album peak [replaygain-album-peak]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_MIMETYPE, "Mime Type (gchararray) [mimetype]"),
 			{ 0, 0, 0 }
 		};
-
+		g_assert ((sizeof (values) / sizeof (values[0]) - 1) == RHYTHMDB_NUM_SAVED_PROPERTIES);
 		etype = g_enum_register_static ("RhythmDBPropType", values);
 	}
 
@@ -1946,19 +2096,19 @@ rhythmdb_unsaved_prop_get_type (void)
 		static const GEnumValue values[] =
 		{
 
-			ENUM_ENTRY (RHYTHMDB_PROP_TITLE_SORT_KEY, "Title sort key (gchararray)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_GENRE_SORT_KEY, "Genre sort key (gchararray)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_ARTIST_SORT_KEY, "Artist sort key (gchararray)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM_SORT_KEY, "Album sort key (gchararray)"),
+			ENUM_ENTRY (RHYTHMDB_PROP_TITLE_SORT_KEY, "Title sort key (gchararray) [title-sort-key]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_GENRE_SORT_KEY, "Genre sort key (gchararray) [genre-sort-key]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_ARTIST_SORT_KEY, "Artist sort key (gchararray) [artist-sort-key]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM_SORT_KEY, "Album sort key (gchararray) [album-sort-key]"),
 
-			ENUM_ENTRY (RHYTHMDB_PROP_TITLE_FOLDED, "Title folded (gchararray)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_GENRE_FOLDED, "Genre folded (gchararray)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_ARTIST_FOLDED, "Artist folded (gchararray)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM_FOLDED, "Album folded (gchararray)"),
-			ENUM_ENTRY (RHYTHMDB_PROP_LAST_PLAYED_STR, "Last Played (gchararray)"),
+			ENUM_ENTRY (RHYTHMDB_PROP_TITLE_FOLDED, "Title folded (gchararray) [title-folded]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_GENRE_FOLDED, "Genre folded (gchararray) [genre-folded]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_ARTIST_FOLDED, "Artist folded (gchararray) [artist-folded]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_ALBUM_FOLDED, "Album folded (gchararray) [album-folded]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_LAST_PLAYED_STR, "Last Played (gchararray) [last-played-str]"),
 			{ 0, 0, 0 }
 		};
-
+		g_assert ((sizeof (values) / sizeof (values[0]) - 1) == RHYTHMDB_NUM_PROPERTIES - RHYTHMDB_NUM_SAVED_PROPERTIES);
 		etype = g_enum_register_static ("RhythmDBUnsavedPropType", values);
 	}
 
@@ -2011,7 +2161,8 @@ void
 rhythmdb_emit_entry_restored (RhythmDB *db, RhythmDBEntry *entry)
 {
 	g_signal_emit (G_OBJECT (db), rhythmdb_signals[ENTRY_RESTORED], 0, entry);
-	if (rhythmdb_entry_get_int (db, entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_SONG) {
+	if (!db->priv->no_update
+	    && rhythmdb_entry_get_int (db, entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_SONG) {
 		rhythmdb_entry_ref_unlocked (db, entry);
 		g_async_queue_push (db->priv->update_queue, entry);
 	}
@@ -2092,3 +2243,33 @@ rhythmdb_compute_status_normal (gint n_songs, glong duration, GnomeVFSFileSize s
 	return ret;
 }
 
+static RBAtomic last_entry_type = {0};
+
+RhythmDBEntryType
+rhythmdb_entry_register_type (void)
+{
+	/* FIXME: does it need locking ? */
+	return rb_atomic_inc (&last_entry_type);		
+}
+
+RhythmDBEntryType rhythmdb_entry_song_get_type (void) 
+{
+	static RhythmDBEntryType song_type = -1;
+       
+	if (song_type == -1) {
+		song_type = rhythmdb_entry_register_type ();
+	}
+
+	return song_type;
+}
+
+RhythmDBEntryType rhythmdb_entry_iradio_get_type (void) 
+{
+	static RhythmDBEntryType iradio_type = -1;
+       
+	if (iradio_type == -1) {
+		iradio_type = rhythmdb_entry_register_type ();
+	}
+
+	return iradio_type;
+}
